package cse378;
import java.util.LinkedList;

import cse378.address.Address;
import cse378.address.PageOffset;
import cse378.address.PhysicalAddress;
import cse378.address.PhysicalPage;
import cse378.address.VirtualAddress;
import cse378.address.VirtualPage;
import cse378.cache.Cache;
import cse378.pagetable.PageTable;
import cse378.pagetable.PageTableEntry;
import cse378.status.Access;
import cse378.status.Component;
import cse378.status.Status;
import cse378.tlb.TLB;


public class MemorySystem {

	static AbstractCache ICache = null;
	static AbstractCache DCache = null;
	static TLB ITLB = null;
	static TLB DTLB = null;
	static PageTable pageTable = null;

	MemorySystem(){ }

	public void initPageTable(int pages){
		pageTable = new PageTable(pages);
	}

	public void initICache(int lines, int sets, int blockSize){
		MemorySystem.ICache = new Cache( Component.ICACHE, lines, sets, blockSize);
	}

	public void initDCache(int lines, int sets, int blockSize){
		MemorySystem.DCache = new Cache( Component.DCACHE, lines,  sets,  blockSize);
	}

	public void initITLB(int lines, int sets){
		MemorySystem.ITLB = new TLB(Component.ITLB, lines, sets);
	}

	public void initDTLB(int lines, int sets){
		MemorySystem.DTLB = new TLB(Component.DTLB,lines,sets);
	}

	/**
	 * Perform address translatio
	 * 
	 * @param tlb	TLB to use (ITLB or DTLB)
	 * @param vaddr VirtualAddress to translate
	 * @param statusList List of the statuses of your accesses
	 * @param isStore If a write is occurring on this address
	 * @return PhysicalPage for vaddr
	 */
	public PhysicalPage translate(TLB tlb, VirtualAddress vaddr,LinkedList<Status> statusList, boolean isStore){
		// Easy Translation
		PhysicalPage page = null;
		Status tlbStatus = null;
		Status pageTableStatus = null;

		// get the page from the pageTable (do even if TLB hits)
		page = (isStore)? pageTable.store(vaddr) : pageTable.load(vaddr);

		// If mapping changed in the pageTable then the TLBs must be kept up to date
		// Handle tlb evictions caused by changes to the pageTable
		if(pageTable.pendingTLBEviction() != null){
			VirtualAddress killAddress = new VirtualAddress(pageTable.pendingTLBEviction(),vaddr.getOffset());
			ITLB.evict(killAddress);
			DTLB.evict(killAddress);
		}
		
		// Now, Check if the TLB has that mapping, fix it otherwise
		if(isStore){
			tlb.write(vaddr,page);
		} else {
			tlb.read(vaddr,page);
		}

		tlbStatus = tlb.getStatus();
		statusList.addLast(tlbStatus);

		pageTableStatus = pageTable.getStatus();
		statusList.addLast(pageTableStatus);

		return page;
	}	

	/** Send a request to fetch an instruction
	 * 
	 * @param vaddr virtual address of the instruction
	 * @return Status of this attempt
	 */
	public LinkedList<Status> fetch(VirtualAddress vaddr){
		PhysicalPage page = null;
		PageTableEntry pte = null;
		LinkedList<Status> statusList = new LinkedList<Status>();
		// Get the physical page
		
		// Construct physical address

		// Read from ICache

		// Check the ICache Status
		Status status = ICache.getStatus();
		statusList.addLast(status);
		
		throw new RuntimeException("Implement Me!");
		
		return statusList; 
	}

	/** Send a request to read from memory
	 * 
	 * @param vaddr virtual address of the instruction
	 * @return Status of this attempt
	 */
	public LinkedList<Status> read(VirtualAddress vaddr){
		PhysicalPage page = null;
		LinkedList<Status> statusList = new LinkedList<Status>();
		// Get the physical page
		
		// Construct physical address

		// Read from the DCache

		// Check the DCache Status

		Status status = DCache.getStatus();
		System.err.println("read: "+vaddr+" ==> "+status);
		statusList.addLast(status);

		throw new RuntimeException("Implement Me!");

		return statusList; 
	}

	/** Send a request to write to memory
	 * 
	 * @param vaddr virtual address of the instruction
	 * @return Status of this attempt
	 */
	public LinkedList<Status> write(VirtualAddress vaddr){
		PhysicalPage page = null;
		LinkedList<Status> statusList = new LinkedList<Status>();
		// Get the physical page
		
		// Construct physical address

		// Write to the DCache

		// Check the DCache Status
		
		// Check the DCache Status
		Status status = DCache.getStatus();
		System.err.println("write: "+vaddr+" ==> "+status);
		statusList.addLast(status);

		throw new RuntimeException("Implement Me!");

		return statusList; 
	}
	
	public String getStats(){
		return "       :\tAccess\tHit\tMiss\tDirtyMiss\n"+
				"DCache: \t"+DCache.getCounts()+"\n"+
				"ICache: \t"+ICache.getCounts()+"\n"+
				"ITLB  : \t"+ITLB.getCounts()+"\n"+
				"DTLB  : \t"+DTLB.getCounts()+"\n"+
				"PageFaults: \tClean: "+pageTable.getCleanMisses()+"\tDirty: "+pageTable.getDirtyMisses();

	}
	
}
