package cse378;
import java.io.*;
import java.util.*;

import cse378.address.VirtualAddress;
import cse378.status.Status;

public class MemorySystemTester {

	static MemorySystem mem = new MemorySystem();

	String inputfile;
	String logfile;

	public static void main(String[] args){
		mem.initPageTable(1);
		mem.initDCache(16, 4, 8);
		mem.initICache(16, 4, 8);
		mem.initDTLB(16, 1);
		mem.initITLB(16, 1);

		int fetchAddress, dataAddress;
		boolean write = false;
		try {
			BufferedReader in = new BufferedReader(new FileReader(args[0]));
			String line = in.readLine();
			String[] fields;
			while (line != null) {
				line = line.replace('[',' ').
						replace(']',' ').
						trim();
				fields = line.split("[ \t]+");
				fetchAddress = Integer.decode(fields[0]);
				LinkedList<Status> fetchStatus = mem.fetch(new VirtualAddress(fetchAddress));
//				updateFetchStats(fetchAddress,fetchStatus);
				if(fields.length > 1){
					if(fields[1].matches("S[BHW]")){
						write = true;
					} else if(fields[1].matches("L[BHW=]")){
						write = false;
					} else {
						throw new RuntimeException("Invalid OP code: "+fields[1]);
					}
					dataAddress = Integer.decode(fields[2]); 
//					System.err.println(((write)? "WRITE to: ": "READ from:")+String.format("%1$08X",dataAddress));
					LinkedList<Status> dataStatus = (write)? mem.write(new VirtualAddress(dataAddress)) : mem.read(new VirtualAddress(dataAddress));
//					updateDataStats(dataAddress,dataStatus,write);			
				}
				line = in.readLine();
			}
			in.close();
		}catch(IOException e){
			throw new RuntimeException(e.getMessage());
		}catch(RuntimeException e){
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		System.err.print("Final Stats\n"+mem.getStats());

	}
	
	protected static void updateFetchStats(int address, LinkedList<Status> stats ){
		String[] objs = { "ITLB", "PAGETABLE", "ICACHE" };
		System.out.println("F "+Integer.toHexString(address)+" ");
		for(Iterator i = stats.iterator(); i.hasNext();){
			Status s = (Status) i.next();
			System.out.println("\t "+s);
		}
	}

	protected static void updateDataStats(int address, LinkedList<Status> stats , boolean write){
		String[] objs = { "DTLB", "PAGETABLE", "DCACHE" };
		System.out.println(((write)?"W ":"R ")+Integer.toHexString(address)+" ");
		for(Iterator i = stats.iterator(); i.hasNext();){
			Status s = (Status) i.next();
			System.out.println("\t "+s);
		}
	}
}


