package cse378;

import cse378.util.BitField;

public abstract class TaggedBlock extends AbstractBlock {

	protected BitField tag;
	protected final byte tagBits;

	public TaggedBlock(int tagBits, int id) {
		super(id);
		assert(tagBits > 0 && tagBits < 33);
		this.tagBits = (byte) tagBits;
	}

	/**getTag returns the tag
	   @return tag of cacheline
	 */
	public BitField getTag() {
		return tag;
	}

	/**writeTag writes the new tag
	 *@param tag new tag for the cacheline
	 */
	public void setTag(BitField tag) {
		this.tag = tag;
	}

	/** Checks this entry for a matching Tag
	 * 
	 * @param tag
	 * @return true if the tags match
	 */
	public boolean hasTag(BitField tag) {
		return this.tag.equals(tag);
	}
	
	/** Update the current entry with a new one
	 * @param tag new Tag
	 */
	public void update(BitField tag){
		this.setDirty(false);
		this.setValid(true);
		this.setTag(tag);
	}
	
	public String toString(){
		return "["+this.id+"] tag = "+getTag()+" D|V = "+((this.dirty)? "1": "0")+"|"+((this.valid)? "1":"0");
	}

}