package cse378.address;

import cse378.util.Bits;


public class PhysicalAddress extends Address {

	/** Construct a VirtualAddress from an int
	 * 
	 * @param raw the integer representing the raw address
	 */
	public PhysicalAddress(int raw){
		this((raw >>> PageOffset.BITWIDTH),Bits.truncate(raw, PageOffset.BITWIDTH));
	}

	/** Construct a PhysicalAddress from an page number and offset
	 *  Address uses Bits.truncateSafe to ensure that page and offset
	 *  both falls in appropriate ranges
	 * @param page int representing the physical page
	 * @param offset int representing the page offset
	 */
	public PhysicalAddress(int page, int offset){
		this(new PhysicalPage(page),new PageOffset(offset));
	}

	public PhysicalAddress(PhysicalPage page, PageOffset offset){
		super(page,offset);
	}
	
	public String toString(){
		return "Physical Address: "+page+" "+super.toString();
	}
	
	public PageOffset getOffset(){
		return (PageOffset) this.offset;
	}
	
	public PhysicalPage getPage(){
		return (PhysicalPage) this.page;
	}
}
