package cse378.address;

import cse378.util.Bits;

public class VirtualAddress extends Address {

	/** Construct a VirtualAddress from an int
	 * 
	 * @param raw the integer representing the raw address
	 */
	public VirtualAddress(int raw){
		this((raw >>> PageOffset.BITWIDTH),Bits.truncate(raw, PageOffset.BITWIDTH));
	}
	
	/** Construct a VirtualAddress from an page number and offset
	 * 
	 * @param page int representing the physical page
	 * @param offset int representing the page offset
	 */
	public VirtualAddress(int page, int offset){
		this(new VirtualPage(page),new PageOffset(offset));
	}

	public VirtualAddress(VirtualPage page, PageOffset offset){
		super(page,offset);
	}

	public PageOffset getOffset(){
		return (PageOffset) this.offset;
	}
	
	public VirtualPage getPage(){
		return (VirtualPage) this.page;
	}

	public String getInfo(){
		return "VA ["+this.toString()+"] = VP ["+Bits.toBinaryString(getPage().value,20)+"] + ["+Bits.toBinaryString(getOffset().value, 12)+"]";
	}
}
