package cse378.pagetable;

import cse378.TaggedBlock;
import cse378.address.PhysicalPage;
import cse378.address.VirtualPage;

/** Represents the mapping between a virtual page and a physical page
 * 
 * @author alaska
 *
 */

public class PageTableEntry extends TaggedBlock{
	protected boolean swapped;
	protected PhysicalPage page;
	static int count = 0;
	
	public PageTableEntry(VirtualPage v, PhysicalPage p){
		super(v.getWidth(),count++);
		setSwapped(false);
		setTag(v);
		setPhysical(p);
	}
	
	/** Get the virtual page represented by this mapping
	 * @return VirtualPage
	 */
	@Override
	public VirtualPage getTag(){
		return (VirtualPage) tag;
	}

	/** Change the virtual page represented by this mapping
	 * @param tag new VirtualPage for this mapping 
	 */
	public void setTag(VirtualPage tag){
		this.tag = tag;
	}

	/** Get the physical page represented by this mapping
	 * @return PhysicalPage 
	 */
	public PhysicalPage getPhysical(){ 
		return this.page; 
	}

	/** Change the physical page represented by this mapping
	 * @param tag new PhysicalPage for this mapping 
	 */
	public void setPhysical( PhysicalPage p ){ 
		this.page = p;
	}
	
	/** Check if this page table entry has been swapped out
	 * @return true is its been swapped out
	 */
	public boolean isSwapped(){
		return this.swapped;
	}

	public void setSwapped(boolean s){ 
		this.swapped = s;
	}
	
	public String toString(){
		return "["+this.id+"] tag = "+getTag()+" D|V|S = "+((this.dirty)? "1": "0")+"|"+((this.valid)? "1":"0")+"|"+((this.swapped)? "1":"0");
	}

}
