package cse378.tlb;
import java.util.Arrays;

import cse378.address.PhysicalPage;
import cse378.address.VirtualAddress;
import cse378.address.VirtualPage;
import cse378.status.Access;
import cse378.status.Component;
import cse378.status.Status;
import cse378.util.BitField;
import cse378.util.Bits;


public class TLB {
	int numLines;
	int numSets;
	TLBLine[] lines;
	
	protected long hits;
	protected long cleanMisses;
	protected long dirtyMisses;
	
	Status stat;
		
	public TLB(Component c, int lines, int sets){
		this.numLines = lines;
		this.numSets = sets;
		this.lines = new TLBLine[this.numLines];
    	// init the status
    	stat = new Status(c);
    	
    	// Allocate the lines
    	Arrays.fill(this.lines, new TLBLine( 20, this.numSets));
	   	
	}
	
	/** Invalidate the TLB entry
	 * 
	 * @param vaddr
	 * @return Invalidated entry
	 */
	public TLBEntry evict(VirtualAddress vaddr){
		TLBEntry te = probe(vaddr,false);
		if(te != null){
			te.setValid(false);
		}
		return te;
	}
	
	protected TLBEntry probe(VirtualAddress vaddr, boolean markRecent){
		this.stat.setAddress(vaddr);
		BitField index = computeIndex(vaddr); // use top 20 bits to probe TLB
		TLBLine theLine = getLine(index);
		TLBEntry theEntry = theLine.probe(vaddr.getPage());
		if(theEntry == null){ // probe didn't find a match
			stat.setClean();
		} else {
			if(markRecent) theLine.setMRU(theEntry);
		}
		return theEntry;
	}
	
	public Status getStatus(){
		return stat;
	}
	
	protected TLBLine getLine(BitField vp){
		return lines[vp.value];
	}
	
	/**Compute the index of a particular location
	*@param location location in memory to be wrote to
	*@return integer that indexes into the lines in the cache
	*/
	protected BitField computeIndex(VirtualAddress location){
		throw new RuntimeException("Implement Me!");
	}

	protected VirtualPage computeTag(VirtualAddress vaddr){
		throw new RuntimeException("Implement Me!");
	}

	public void read(VirtualAddress vaddr, PhysicalPage page){
		throw new RuntimeException("Implement Me!");
	}
	
	public void write(VirtualAddress vaddr, PhysicalPage page){
		throw new RuntimeException("Implement Me!");
	}
	
	 public String getCounts(){
		 long accesses = this.hits + this.cleanMisses + this.dirtyMisses;
		 return accesses+"\t"+this.hits+"\t"+this.cleanMisses+"\t"+this.dirtyMisses;
	 }
}
