package cse378.util;
/**
 * An abstract class representing a fixed-width bit field
 * @author alaska
 *
 */
public class BitField {
	protected final int width; // the width of this BitField
	public final int value;
	
	public BitField(int val, int width){
		this.width = width;
		int tmp = Bits.truncateSafe(val, width+1);
		this.value = tmp;
	}
	
	public int toInt(){
		return Bits.truncate(value, width);
	}
	public int getWidth(){
		return width;
	}
	
	public String toString(){
		return Integer.toString(value);
	}
	
	public String toBinaryString(){
		return Bits.toBinaryString(value,width);
	}
	
	public boolean equals(Object o){
		if(o == null) return false;
		if(o instanceof BitField){
			BitField bf = (BitField) o;
			return (bf.value == value) && (bf.width == width); 
		}
		return false;
	}
	
	public BitField insert(BitField b){
		if(b.width >= width) return b;
		int mask = ~((1 << b.width) - 1);
		int masked = this.value & mask;
 		BitField nb = new BitField((masked|b.value),width);
 		return nb;
	}
	
	public BitField shiftLeft(int i){
		assert(i >= 0 && i <= width);
		int nw = width - i;
		return new BitField(value,(byte) nw);		
	}
	
	public int hashCode(){
		return width ^ value;
	}
	
	public BitField extract(int start, int len){
		assert((start > 0) && (len > 0));
		assert((start + len) < width);
		return new BitField(value,len);
	}
}
