package cse378.util;

public class Bits {
	public Bits(){ };

	/** compute the base 2 log of an integer
	 * @param num
	 * @return b such that 2^b == num (or close)
	 */
	public static byte log2(int num){
		byte i = 0;
		while(num > 1){
			num = num >> 1;
			i++;
		}
		return i;
	}

	/** extract the value of a bitfield
	 * 
	 * @param num 
	 * @param start first bit to include
	 * @param width	number of bits to include
	 * @return bits [end:start] of num as an int
	 */
	public static int extractSafe(int num, int start, int width){
		int mask = (1 << width) - 1; // compute int with end-start 1s  
		num >>>= start; // shift left until bit start is at position 0
		num &= mask;   // turn off all bits past end
		return num;
	}

	/** extract the value of a bitfield,
	 * 
	 * @param num 
	 * @param start first bit to include
	 * @param width	number of bits to include
	 * @return bits [end:start] of num as an int
	 */
	public static int extract(int num, int start, int width){
		int mask = (1 << width) - 1; // compute int with end-start 1s  
		num >>>= start; // shift left until bit start is at position 0
		num &= mask;   // turn off all bits past end
		return num;
	}
	
	/** extract the value of a bitfield
	 * 
	 * @param num 
	 * @param start first bit to include
	 * @param end	last bit to include
	 * @return bits [end:start] of num as an int
	 */
	public static int mask(int num, int maskBits, int start){
		int mask = ((1 << maskBits) - 1); // 2^maskBits - 1 (i.e.) 3 bits: 2^3-1 = 7 (0111) 
		int field = num >>> start; // drop all bits before the field
		return (field & mask); // mask off the high bits
	}
	
	public static int truncate(int num, int bitWidth){
		return Bits.extract(num,0,bitWidth-1);
	}
	
	public static int truncateSafe(int num, int bitWidth){
		assert(Integer.numberOfLeadingZeros(num) >= (32-bitWidth));
		return Bits.extract(num,0,bitWidth-1);
	}

	
	public static String toHexString(int i){
		return String.format("%1$8s",Integer.toHexString(i)).replace(' ','0');
	}
	
	public static String toHexString(int i, int width){
		return toUnsignedString(i,2).substring(8-width);
	}
	
	public static String toBinaryString(int i){
		return String.format("%1$32s",Integer.toBinaryString(i)).replace(' ','0');
//		return toUnsignedString(i,1);
	}
	
	public static String toBinaryString(int i, int width){
		return String.format("%1$32s",Integer.toBinaryString(i)).replace(' ','0').substring(32-width);
		//		return toUnsignedString(i,1);
	}
    /**
     * Convert the integer to an unsigned number.
     */
	private static String toUnsignedString(int i, int shift) {
		final char[] digits = {
				'0' , '1' , '2' , '3' , '4' , '5' , '6' , '7' ,
				'8' , '9' , 'a' , 'b' , 'c' , 'd' , 'e' , 'f' }; 
		char[] buf = new char[32];
		
		int charPos = 32;
		int radix = 1 << shift;
		int mask = radix - 1;
		for(int j = 0; j < (32 >>> shift); j++){
			buf[--charPos] = digits[i & mask];
			i >>>= shift;
		}
		return new String(buf);
	}
	
}
